-- Mannoroth Assist --

local ENCOUNTERID_Mannoroth = 1795
local SPELLID_MannorothsGaze = 181597
local DEBUFF_APPLY_DELAY_MAX = 0.5
local NUM_PLAYERS_ASSIST = 2 --number of players needed to assist/soak the targeted player's debuff

local UnitGUID, GetCurrentMapDungeonLevel, GetCurrentMapZone, GetPlayerMapPosition = UnitGUID, GetCurrentMapDungeonLevel, GetCurrentMapZone, GetPlayerMapPosition

local frame = CreateFrame("Frame", "MannorothAssistFrame", UIParent)
local frameEvents = {};

--/run MannorothAssist_Debug("debug")

--774: Rejuvenation
--77761: bear roar
--/run MannorothAssist_Debug("newspell",774);MannorothAssist_Debug("startcombat")
--/run MannorothAssist_Debug("newspell",77761);MannorothAssist_Debug("startcombat")
--/run MannorothAssist_Debug("endcombat")
local isDebug = false
local function debugLog(...)
  if isDebug then
    print("MannorothAssist", ...)
  end
end
function MannorothAssist_Debug(msg, i)
  if msg == "debug" then
    isDebug = not isDebug
    print("MannorothAssist: debug", isDebug)
  elseif msg == "eid" then
    if type(i) == "number" then
      ENCOUNTERID_Mannoroth = i
      debugLog("neweid =", ENCOUNTERID_Mannoroth, EJ_GetEncounterInfo(i))
    end
  elseif msg == "startcombat" then
    frameEvents["ENCOUNTER_START"](frame, ENCOUNTERID_Mannoroth)
  elseif msg == "endcombat" then
    frameEvents["ENCOUNTER_END"](frame)
  elseif msg == "newspell" then
    if type(i) == "number" then
      SPELLID_MannorothsGaze = i
      debugLog("newspell =", SPELLID_MannorothsGaze, GetSpellInfo(i))
    end
  end
end

function frameEvents:ENCOUNTER_START(encounterID, encounterName, difficultyID, raidSize)
  --print(encounterID, encounterName, difficultyID, raidSize)
  if encounterID == ENCOUNTERID_Mannoroth then
    debugLog("encounter started!")
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    
    --map scale stuff
    local n, a1, b1, c1, d1 = GetCurrentMapDungeonLevel()
    if not (a1 and b1 and c1 and d1) then
      n, a1, b1, c1, d1 = GetCurrentMapZone()
    end
    if not (a1 and b1 and c1 and d1) then
      frame.movement_mapScaleXSquare = 0
      frame.movement_mapScaleYSquare = 0
    else
      frame.movement_mapScaleXSquare = (c1 - a1)^2
      frame.movement_mapScaleYSquare = (d1 - b1)^2
    end
  end
end

function frameEvents:ENCOUNTER_END()
  debugLog("encounter ended.")
  self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function frameEvents:COMBAT_LOG_EVENT_UNFILTERED(...)
  local _, event, _, _, _, _, _, destGUID = select(1, ...)
  if event == "SPELL_AURA_APPLIED" then
    local spellId = select(12, ...) --from prefix SPELL
    if spellId == SPELLID_MannorothsGaze then
      if (self.waitingForDebuffs) and ((destGUID:sub(1, 7)) == "Player-") then
        debugLog("new debuff on player!")
        self:addDebuffedPlayer(destGUID)
      end
    end
  elseif event == "SPELL_AURA_REMOVED" then
    local spellId = select(12, ...) --from prefix SPELL
    if spellId == SPELLID_MannorothsGaze then
      --if (self.waitingForDebuffs) and ((destGUID:sub(1, 7)) == "Player-") then
      if ((destGUID:sub(1, 7)) == "Player-") then
        debugLog("debuff removed from player.")
        self:removeDebuffedPlayer(destGUID)
      end
    end
  end
end

frame:SetScript("OnEvent", function(self, event, ...)
  frameEvents[event](self, ...)
end);

do --init
  frame.debuffCounter = 0
  frame.debuffedPlayers = {};
  frame.waitingForDebuffs = true
  frame.myGUID = UnitGUID("player")
  
  frame.movement_mapScaleXSquare = 0
  frame.movement_mapScaleYSquare = 0
  frame.movement_playerPositions = {};
  
  frame.doTheMath = function(self)
    debugLog("marking players now")
    local nextTargetIcon = 1
    local guid
    local unitId
    
    --map stuff
    for i = 1, MAX_PARTY_MEMBERS do
      unitId = "raid"..i
      guid = UnitGUID(unitId) or nil
      if guid then
        local px, py = GetPlayerMapPosition(unitId)
        self.movement_playerPositions[unitId] = {
          ["X"] = px;
          ["Y"] = py;
          ["hasDebuff"] = self.debuffedPlayers[guid] and true;
          ["isAssisting"] = false;
          ["nearPlayers"] = {}
        };
      end
    end
    
    for i = 1, MAX_PARTY_MEMBERS do
      unitId = "raid"..i
      guid = UnitGUID(unitId) or ""
      if self.debuffedPlayers[guid] then
        debugLog("  debuffed player found with unitId", unitId)
        self.debuffedPlayers[guid] = unitId --cache unitId
        SetRaidTarget(unitId, nextTargetIcon)
        
        if guid == self.myGUID then
          --/say
          --SendChatMessage("{rt"..nextTargetIcon.."}", "SAY")
        end
        
        --map stuff
        for j = 1, MAX_PARTY_MEMBERS do
          if i ~= j then
            local unitId2 = "raid"..j
            if (self.movement_playerPositions[unitId2]) and (not self.movement_playerPositions[unitId2].hasDebuff) and (not self.movement_playerPositions[unitId2].isAssisting) then
              local distance = sqrt((self.movement_playerPositions[unitId].X-self.movement_playerPositions[unitId2].X)^2*self.movement_mapScaleXSquare + (self.movement_playerPositions[unitId].Y-self.movement_playerPositions[unitId2].Y)^2*self.movement_mapScaleYSquare)
              debugLog("    distance to not-debuffed player", unitId2, "is", distance, "yards")
              table.insert(self.movement_playerPositions[unitId].nearPlayers, {unitId2, distance})
            end
          end
        end
        table.sort(self.movement_playerPositions[unitId].nearPlayers, function(a,b)return a[2]<b[2]end)
        
        local assistingPlayers = "{rt"..nextTargetIcon.."}"
        for j = 1, NUM_PLAYERS_ASSIST do
          if self.movement_playerPositions[unitId].nearPlayers[j] then
            local name = UnitName(self.movement_playerPositions[unitId].nearPlayers[j][1])
            assistingPlayers = assistingPlayers.." "..name
            self.movement_playerPositions[self.movement_playerPositions[unitId].nearPlayers[j][1]].isAssisting = true
          end
        end
        --if guid == self.myGUID then
          SendChatMessage(assistingPlayers, "RAID")
        --end
        
        nextTargetIcon = nextTargetIcon + 1
      end
    end
  end
  frame.addDebuffedPlayer = function(self, guid)
    debugLog("addDebuffedPlayer(guid =", guid, ")")
    self.debuffedPlayers[guid] = true
    if self.debuffCounter == 0 then
      self:startTimer()
    end
    self.debuffCounter = self.debuffCounter + 1
  end
  frame.removeDebuffedPlayer = function(self, guid)
    debugLog("removeDebuffedPlayer(guid =", guid, ")")
    local unitId = self.debuffedPlayers[guid]
    if unitId then
      debugLog("  unitId found, removing...")
      SetRaidTarget(unitId, 0)
      
      self.debuffedPlayers[guid] = nil
      self.debuffCounter = self.debuffCounter - 1
      if self.debuffCounter == 0 then
        debugLog("  no more players")
        self.debuffedPlayers = {};
        self.movement_playerPositions = {};
        self.waitingForDebuffs = true
      end
    end
  end
  
  frame.tslU = -1
  frame.interval = DEBUFF_APPLY_DELAY_MAX
  
  frame.NoOp = function() end
  frame.RealOnUpdate = function(self, elapsed)
    self.tslU = self.tslU + elapsed
    if self.tslU >= self.interval then
      debugLog("timer finished")
      self.tslU = -1
      self.OnUpdate = self.NoOp
      
      self.waitingForDebuffs = false
      self:doTheMath()
    end
  end
  frame.OnUpdate = frame.NoOp
  frame:SetScript("OnUpdate", function(self, elapsed)
    self.OnUpdate(self, elapsed)
  end);
  
  frame.startTimer = function(self)
    debugLog("startTimer()")
    self.tslU = 0
    self.OnUpdate = self.RealOnUpdate
  end
  
  frame:RegisterEvent("ENCOUNTER_START")
  frame:RegisterEvent("ENCOUNTER_END")
end
